/****************************************************************************
*
*     Module Filename:  Krnl_Vis.cpp
*     Module Name:      Kernel Visibility API
*     Description:      Visibility API for the SimSoup Kernel
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include  "globldef.h"
#include  "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Kernel_Manager ---------------------------------------------------------------------------------
#include  "Krnl_Vis.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  Chemistry  Subsystem  --------------------------------------------------------------------------
#include  "Chemistry.h"
#include  "Molecule_Type.h"
#include  "Interaction_Type.h"
//----------------------------------------------------------------------------------------------------------------

//--------------  Reactor  Subsystem  ----------------------------------------------------------------------------
#include  "Reactor.h"
#include  "Realised_Molecule_Type.h"
#include  "Realisable_Interaction_Type.h"
//----------------------------------------------------------------------------------------------------------------

//----------------Simulation Subsystem ---------------------------------------------------------------------------
#include  "Scenario_Manager.h"
#include  "Scenario_Series.h"
#include  "Scenario.h"
#include  "Action_Request.h"
//----------------------------------------------------------------------------------------------------------------

//---------------- Kernel Manager Subsystem ----------------------------------------------------------------------
#include  "Simulation_Scheduler.h"
//----------------------------------------------------------------------------------------------------------------


Reactor*    Krnl_Vis::Reactor_Ptr = 0;
Chemistry*  Krnl_Vis::Chemistry_::Chemistry_Ptr = 0;
Scenario_Manager*  Krnl_Vis::Scenario_Manager_Ptr = 0;
Simulation_Scheduler* Krnl_Vis::Sim_Scheduler_Pt_ = 0;

const std::string Int_Type_Categ_Names[3] = {"Construction",  "Fission", "Transformation"};

//---------------------------------------------------------------------------------------------------------------

// int  Krnl_Vis::Action_Request_::Get_Number(Action_Request*  Action_Request_Ptr)
// 
// //  Get the number of actions associated with the specified Action_Request
// 
// {
//     return Action_Request_Ptr->Get_Number();
// }
//---------------------------------------------------------------------------------------------------------------

void  Krnl_Vis::Action_Request_::Get_Params(Action_Request* Action_Request_Ptr, list<string>& Param_List)

//  Get the parameters for the Get_Number_Of_Molecule_Typesspecified Action_Request

{
    TRACE("Krnl_Vis::Action_Request_::Get_Params");
    Action_Request_Ptr->Get_Params(Param_List);
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Action_Request_::Get_Repeat_Interval(Action_Request*  Action_Request_Ptr)

//  Get the repeat interval for the specified Action Request

{
    return      Action_Request_Ptr->Get_Repeat_Interval();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Action_Request_::Get_Repeat_Req_Flag(Action_Request*  Action_Request_Ptr)

//  Get the repeat request flag for the specified Action_Request

{
    return Action_Request_Ptr->Get_Repeat_Req_Fl();
}
//-----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Scenario_::Get_Statistics_Collection_Interval()

//  Get the statistics collection interval for the current Scenario

{
    return Get_Curr_Scenario_Ptr()->Get_Statistics_Collection_Interval();
}
//----------------------------------------------------------------------------------------------------------------

string  Krnl_Vis::Action_Request_::Get_Subject(Action_Request*  Action_Request_Ptr)

//  Get the Subject for the current Action_Request

{
    return Action_Request_Ptr->Get_Subject();
}
//---------------------------------------------------------------------------------------------------------------

int  Krnl_Vis::Action_Request_::Get_Time(Action_Request*  Action_Request_Ptr)

//  Get the action Time for the current Action Request

{
    return Action_Request_Ptr->Get_Time();
}
//----------------------------------------------------------------------------------------------------------------

string  Krnl_Vis::Action_Request_::Get_Type(Action_Request*  Action_Request_Ptr)

//  Get the Type of the current Action_Request

{
    return Action_Request_Ptr->Get_Type();
}
//----------------------------------------------------------------------------------------------------------------

void Krnl_Vis::Chemistry_::Clear()

//  Clear the Chemistry

{
    Chemistry_Ptr->Clear();
}
//---------------------------------------------------------------------------------------------------------------

Interaction_Type* Krnl_Vis::Chemistry_::Get_Current_Int_Type_Ptr()

//  Get a pointer to the Chemistry's current Interaction Type

{
    return  Chemistry_Ptr->Get_Current_Int_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type* Krnl_Vis::Chemistry_::Get_Current_Mol_Type_Ptr()

//  Get a pointer to the Chemistry's current Molecule Type

{
    return Chemistry_Ptr->Get_Current_Mol_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type*  Krnl_Vis::Chemistry_::Get_Molecule_Type_Ptr(int Node_Index)

//  Get a pointer to the specified Molecule Type

{
    return Chemistry::Get_MolType_Pt(Node_Index);
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Chemistry_::Get_Number_Of_Molecule_Types()

//  Get the number of Molecule Types in the Chemistry

{
    return Chemistry::Get_Num_MolTypes();
}
//----------------------------------------------------------------------------------------------------------------

void Krnl_Vis::Chemistry_::Initialise()

//  Initialise the Chemistry

{
    Chemistry_Ptr->Initialise();
}
//---------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Chemistry_::Iter_To_First_Int_Type()

//  Set the Chemistry's current Interaction Type to first

{
    return Chemistry_Ptr->Iter_To_First_Int_Type();
}
//----------------------------------------------------------------------------------------------------------------


bool Krnl_Vis::Chemistry_::Iter_To_Next_Int_Type()

//  Set the Chemistry's current Interaction Type to next

{
    return Chemistry_Ptr->Iter_To_Next_Int_Type();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Chemistry_::Iter_To_First_Mol_Type()

//  Set the Chemistry's current Molecule Type to the first
{
    return Chemistry_Ptr->Iter_To_First_Mol_Type();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Chemistry_::Iter_To_Next_Mol_Type()

//  Set the Chemistry's current Molecule Type to the next
{
    return Chemistry_Ptr->Iter_To_Next_Mol_Type();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::Construction_::Get_Max_Rand_Rate_Constant()

//  Get the maximum rate constant to be randomly assigned to Constructions

{
    return Construction::Get_Max_Rand_Rate_Constant();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::Fission_::Get_Max_Rand_Rate_Constant()

//  Get the maximum rate constant to be randomly assigned to Fissions

{
    return Fission::Get_Max_Rand_Rate_Constant();
}
//----------------------------------------------------------------------------------------------------------------

std::string   Krnl_Vis::IntType::Convert_Int_Type_Categ_To_Text(Interaction_Type_Enum Interaction_Type_Enumr)

//  Convert the Interaction Type code to text

{
    return Int_Type_Categ_Names[Interaction_Type_Enumr];
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::IntType::Get_Activated_Complex_Energy(Interaction_Type* IntType_Ptr)

//  Return the Activated Complex Energy for the Interaction Type

{
    return IntType_Ptr->Get_Activated_Complex_Energy();
}
//----------------------------------------------------------------------------------------------------------------

Interaction_Type_Enum   Krnl_Vis::IntType::Get_Category(Interaction_Type*  IntType_Ptr)

//  Determine the category for the specified Interaction Type

{
    return IntType_Ptr->Get_Category();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::IntType::Get_Forward_Activation_Energy(Interaction_Type* IntType_Ptr)

//  Return the forward activation energy for the Interaction Type

{
    return  IntType_Ptr->Get_Forward_Activation_Energy();
}
//----------------------------------------------------------------------------------------------------------------

double  Krnl_Vis::IntType::Get_Forward_Rate_Constant(Interaction_Type* IntType_Ptr)

//  Return the Forward Rate Constant for the Interaction type

{
    return IntType_Ptr->Get_Forward_Rate_Constant();
}
//----------------------------------------------------------------------------------------------------------------


double Krnl_Vis::IntType::Get_Heat_Release(Interaction_Type* IntType_Ptr)

//  Get the heat released by one Interaction ogf the Type

{
    return IntType_Ptr->Get_Heat_Release();
}
//----------------------------------------------------------------------------------------------------------------


int Krnl_Vis::IntType::Get_Identifier(Interaction_Type* IntType_Ptr)

//  Get the Identifier for the specified Interaction_Type

{
    return IntType_Ptr->Get_Identifier();
}
//----------------------------------------------------------------------------------------------------------------

void  Krnl_Vis::IntType::Get_Int_Type_Characteristics(Interaction_Type*  IntType_Ptr,
                                                      int& Interaction_Type_Identifier,
                                                      Interaction_Type_Enum&  Interaction_Type_Enumr,
                                                      int& Reactant1_Identifier,
                                                      int& Reactant2_Identifier,
                                                      int& Product1_Identifier,
                                                      int& Product2_Identifier,
                                                      int& Catalyst_Identifier)

//  Get the characteristics of the specified Interaction Type

{
    IntType_Ptr->Get_Int_Type_Characteristics(Interaction_Type_Identifier,
                                              Interaction_Type_Enumr,
                                              Reactant1_Identifier,
                                              Reactant2_Identifier,
                                              Product1_Identifier,
                                              Product2_Identifier,
                                              Catalyst_Identifier);
}
//----------------------------------------------------------------------------------------------------------------

void Krnl_Vis::IntType::Get_Int_Type_Inputs_No_Dups(Interaction_Type* IntType_Ptr,
                                                    VCSList* TList__Of_Interaction_Inputs_Ptr)

//  Get the inputs for the specified Interaction Type, with no duplicates

{
    IntType_Ptr->Get_Int_Type_Inputs_No_Dups(TList__Of_Interaction_Inputs_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

void Krnl_Vis::IntType::Get_Int_Type_Products_With_Dups(Interaction_Type* IntType_Ptr,
                                                        VCSList* TList__Of_Interaction_Inputs_Ptr)

//  Get the products of the specified Interaction Type, with duplicates

{
    IntType_Ptr->Get_Int_Type_Products_With_Dups(TList__Of_Interaction_Inputs_Ptr);
}
//----------------------------------------------------------------------------------------------------------------


int Krnl_Vis::IntType::Get_Product1_MolType_Id(Interaction_Type*  IntType_Ptr)

//  Return the identifier for Product 1 of the specified Interaction Type

{
    return IntType_Ptr->Get_Product1_MolType_Ptr()->Get_Identifier();
}
//----------------------------------------------------------------------------------------------------------------
int Krnl_Vis::IntType::Get_Product2_MolType_Id(Interaction_Type*  IntType_Ptr)

//  Return the identifier for Product 2 of the specified Interaction Type

{
    Molecule_Type*  MolType_Ptr = IntType_Ptr->Get_Product2_MolType_Ptr();
    if (MolType_Ptr)
    THEN
        return MolType_Ptr->Get_Identifier();
    ELSE
        Assert (true == false, "Krnl_Vis::IntType::Get_Product2_MolType_Id - Invalis Mol Type Ptr");
    return 1;  //  Avoids compiler warning
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------
Molecule_Type*  Krnl_Vis::IntType::Get_Product1_MolType_Ptr(Interaction_Type*  IntType_Ptr)

//  Return a pointer to Product 1 for the specified Interaction Type

{
    return IntType_Ptr->Get_Product1_MolType_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type*  Krnl_Vis::IntType::Get_Product2_MolType_Ptr(Interaction_Type*  IntType_Ptr)

//  Return a pointer to Product 2 for the specified Interaction Type

{
    return IntType_Ptr->Get_Product2_MolType_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

string Krnl_Vis::IntType::Get_Product2_MolType_Text(Interaction_Type*  IntType_Ptr)

//  Return a text string for the Product 2 Identifier, or null if there is no Product 2

{
    Molecule_Type*  MolType_Ptr = Get_Product2_MolType_Ptr(IntType_Ptr);
    if(MolType_Ptr)
    THEN
        return To_String(MolType_Ptr->Get_Identifier());
    ELSE
        return string("");
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::IntType::Get_Reactant1_MolType_Id(Interaction_Type*  IntType_Ptr)

//  Get the Molecule Type Identifier for Reactant 1 of the specified Interaction Type

{
    return IntType_Ptr->Get_Reactant1_MolType_Ptr()->Get_Identifier();
}
//---------------------------------------------------------------------------------------------------------------

Molecule_Type*  Krnl_Vis::IntType::Get_Reactant1_MolType_Ptr(Interaction_Type*  IntType_Ptr)

//  Get the Molecule Type Pointer for Reactant 1 of the specified Interaction Type

{
    return IntType_Ptr->Get_Reactant1_MolType_Ptr();
}
//---------------------------------------------------------------------------------------------------------------

int Krnl_Vis::IntType::Get_Reactant2_MolType_Id(Interaction_Type*  IntType_Ptr)

//  Get the Molecule Type Identifier for Reactant 2 of the specified Interaction Type

{
    Molecule_Type*  MolType_Ptr = IntType_Ptr->Get_Reactant2_MolType_Ptr();
    if (MolType_Ptr)
    THEN
        return MolType_Ptr->Get_Identifier();
    ELSE
        Assert (true == false, "Krnl_Vis::IntType::Get_Reactant2_MolType_Id - Invalis Mol Type Ptr");
        return  1;  // To avoid compiler warning
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type*  Krnl_Vis::IntType::Get_Reactant2_MolType_Ptr(Interaction_Type*  IntType_Ptr)

//  Get the Molecule Type Pointer for Reactant 2 of the specified Interaction Type

{
    return IntType_Ptr->Get_Reactant2_MolType_Ptr();
}
//---------------------------------------------------------------------------------------------------------------

string Krnl_Vis::IntType::Get_Reactant2_MolType_Text(Interaction_Type*  IntType_Ptr)

//  Return a text string for the Reactant 2 Identifier, or null if there is no Product 2

{
    Molecule_Type*  MolType_Ptr = Get_Reactant2_MolType_Ptr(IntType_Ptr);
    if(MolType_Ptr)
    THEN
        return To_String(MolType_Ptr->Get_Identifier());
    ELSE
        return string("");
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------




double Krnl_Vis::IntType::Get_Reverse_Activation_Energy(Interaction_Type* IntType_Ptr)

//  Return the reverse activation energy for the Interaction Type

{
    return IntType_Ptr->Get_Reverse_Activation_Energy();
}
//----------------------------------------------------------------------------------------------------------------

double  Krnl_Vis::IntType::Get_Reverse_Rate_Constant(Interaction_Type* IntType_Ptr)

//  Return the Reverse Rate Constant for the Interaction type

{
    return IntType_Ptr->Get_Reverse_Rate_Constant();
}
//----------------------------------------------------------------------------------------------------------------

Interaction_Type* Krnl_Vis::MolType::Get_Current_IntType_With_MolType_As_Input(Molecule_Type*  MolType_Ptr)

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Input.Get_Current_Item();
}
//----------------------------------------------------------------------------------------------------------------

Interaction_Type* Krnl_Vis::MolType::Get_Current_IntType_With_MolType_As_Product(Molecule_Type*  MolType_Ptr)

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Product.Get_Current_Item();
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::MolType::Get_Identifier(Molecule_Type*  MolType_Ptr)

//  Get the Identifier for the specified Molecule Type

{
    return MolType_Ptr->Get_Identifier();
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::MolType::Get_Mass(Molecule_Type*  MolType_Ptr)

{
    return MolType_Ptr->Get_Mass();
}
//----------------------------------------------------------------------------------------------------------------
int Krnl_Vis::MolType::Get_Max_Mass()

//  Get the maximum mass for Molecule Types

{
    return Molecule_Type::Get_Max_Molecule_Mass();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::MolType::Get_Potential_Energy(Molecule_Type*  MolType_Ptr)

{
    return MolType_Ptr->Get_Potential_Energy();
}
//---------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::MolType::Iter_To_First_IntType_With_MolType_As_Input(Molecule_Type* MolType_Ptr )

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_First();
}
//---------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::MolType::Iter_To_Next_IntType_With_MolType_As_Input(Molecule_Type* MolType_Ptr )

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_Next();
}
//---------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::MolType::Iter_To_First_IntType_With_MolType_As_Product(Molecule_Type* MolType_Ptr )

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Product.Iter_To_First();
}
//---------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::MolType::Iter_To_Next_IntType_With_MolType_As_Product(Molecule_Type* MolType_Ptr )

{
    return MolType_Ptr->List_Of_Int_Types_With_MolType_As_Product.Iter_To_Next();
}
//---------------------------------------------------------------------------------------------------------------

void Krnl_Vis::Reactor_::Clear()

//  Clear the Reactor

{
    Reactor_Ptr->Clear();
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Reactor_::Get_Num_Mols()

//  Get the number of Molecutes in the Reactor

{
    return Reactor_Ptr->Get_Num_Mols_In_Reactor();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::Reactor_::Get_Leakage_Proportion()

//  Get the proportion of Molecules to leak from the Reactor at each timestep

{
    return Reactor_Ptr->Get_Reactor_Leakage_Proportion();
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Reactor_::Get_Time()

//  Get the Reactor Time

{
    return Reactor_Ptr->Get_Reactor_Time();
}
//----------------------------------------------------------------------------------------------------------------

int Krnl_Vis::Reactor_::Get_Volume()

{
    return Reactor_Ptr->Get_Reactor_Volume();
}
//----------------------------------------------------------------------------------------------------------------

void Krnl_Vis::Reactor_::Initialise()

//  Initialise the Reactor

{
    Reactor_Ptr->Initialise();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Reactor_::Iter_To_First_R_Int_Type()

//  Select the Reactor's first Realisable Interaction Type

{
    return  Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Iter_To_First_R_Int_Type();
}
//---------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Reactor_::Iter_To_Next_R_Int_Type()

//  Select the Reactor's next Realisable Interaction Type

{
    return  Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Iter_To_Next_R_Int_Type();
}
//---------------------------------------------------------------------------------------------------------------

Realisable_Interaction_Type*  Krnl_Vis::Reactor_::Get_Current_R_Int_Type_Ptr()

//  Get a pointer to the Reactor's current Realisable Interaction Type

{
    return  Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Get_Current_R_Int_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Reactor_::Iter_To_First_R_Mol_Type()

//  Set the Reactor's current Realised Molecule Type to te first

{
    return Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Iter_To_First_R_Mol_Type();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Reactor_::Iter_To_Next_R_Mol_Type()

//  Set the Reactor's current Realised Molecule Type to the next

{
    return Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Iter_To_Next_R_Mol_Type();
}
//----------------------------------------------------------------------------------------------------------------

Realised_Molecule_Type*  Krnl_Vis::Reactor_::Get_Current_R_Mol_Type_Ptr()

//  Get a pointer to the Reactor's current Realisable Molecule Type

{
    return  Reactor::Get_Reactor__Ptr()->Get_Iterators_Ptr()->Get_Current_R_Mol_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

Interaction_Type*  Krnl_Vis::Realisable_IntType::Get_Interaction_Type_Ptr(Realisable_Interaction_Type*  Realisable_IntType_Ptr)

//  Get a pointer to the specified R Int Types Interaction

{
    return Realisable_IntType_Ptr->Get_Interaction_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

bool Krnl_Vis::Realisable_IntType::Get_Unrealisable(Realisable_Interaction_Type* Realisable_IntType_Ptr)

//  Get the Realisable / Unrealisable status of the specified Realisable_Interaction_Type

{
    return Realisable_IntType_Ptr->Get_Unrealisable();
}
//----------------------------------------------------------------------------------------------------------------

double Krnl_Vis::Realisable_IntType::Get_Interaction_Rate(Realisable_Interaction_Type* Realisable_IntType_Ptr)

{
    return   Realisable_IntType_Ptr->Get_Interaction_Rate();
}
//---------------------------------------------------------------------------------------------------------------- 

double Krnl_Vis::Realisable_IntType::Update_Interaction_Rates(Realisable_Interaction_Type* Realisable_IntType_Ptr,
                                                               int Reactor_Volume)

//  See called function

{
    return Realisable_IntType_Ptr->Update_Interaction_Rates(Reactor_Volume);
}
//----------------------------------------------------------------------------------------------------------------

Molecule_Type*  Krnl_Vis::R_MolType::Get_Molecule_Type_Ptr(Realised_Molecule_Type*  R_MolType_Ptr)

//  Get a pointer to the Molecule Type for the specified Realised Molecule Type

{
    return R_MolType_Ptr->Get_Molecule_Type_Ptr();
}
//----------------------------------------------------------------------------------------------------------------

int   Krnl_Vis::R_MolType::Get_Num_Of_Mols_In_Reactor(Realised_Molecule_Type*  R_MolType_Ptr)

//  Get the number of Molecules of the specified type

{
    return R_MolType_Ptr->Get_Num_Of_Mols_In_Reactor();
}
//----------------------------------------------------------------------------------------------------------------

Action_Request*  Krnl_Vis::Scenario_::Get_Current_Action_Req_Ptr(Scenario*  Scenario_Ptr)

//  Get a pointer to the current Action_Request for the specified Scenario

{
    return Scenario_Ptr->Get_Curr_Scen_ActReq_Pt();
}

//--------------------------------------------------------------------------------------------------------------

Scenario*  Krnl_Vis::Scenario_::Get_Curr_Scenario_Ptr()

//  Get the a pointer to the current Scenario for the current Scenario Series
//  Return zero if there is no Scenario Series

{
    Scenario_Series* Series_Pt = Scenario_Manager::Get_Ptr()->Get_Curr_Scenario_Series_Ptr();
    if (not Series_Pt) return 0;
    Scenario* Scen_Pt = Series_Pt->Get_Curr_Scenario_Ptr();
    Assert(Scen_Pt not_eq 0, "Krnl_Vis::Scenario_::Get_Curr_Scenario_Ptr() - Scenario Series has no current Scenario");
    return Scen_Pt;
}
//----------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::Scenario_::Iter_To_First_Action_Request(Scenario*  Scenario_Ptr)

//  Go to the first Action Request for the specified Scenario

{
    return  Scenario_Ptr->Iter_First_Scen_ActReq();
}
//----------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::Scenario_::Iter_To_Next_Action_Request(Scenario*  Scenario_Ptr)

//  Go to the next Action Request for the specified Scenario

{
    return  Scenario_Ptr->Iter_Next_Scen_ActReq();
}
//----------------------------------------------------------------------------------------------------------------

bool  Krnl_Vis::Simulation_Scheduler_::SimProc_IsEnabled()

//  Return whether or not the simulation processing is enabled

{
    return Sim_Scheduler_Pt_->SimProc_IsEnabled();
}
//---------------------------------------------------------------------------------------------------------------- 

void  Krnl_Vis::Initialise_API()

//  Initialise the pointers to the singleton objects referenced by the API

{
    Assert(Chemistry_::Chemistry_Ptr == 0,"Krnl_Vis::Initialise - Chemistry_Ptr has already been Initialised");
    Chemistry_::Chemistry_Ptr = Chemistry::Get_Chemistry__Ptr();

    Assert(Reactor_Ptr == 0,"Krnl_Vis::Initialise - Reactor_Ptr has already been Initialised");
    Reactor_Ptr = Reactor::Get_Reactor__Ptr();

    Assert(Scenario_Manager_Ptr == 0,"Krnl_Vis::Initialise - Scenario_Manager_Ptr has already been Initialised");
    Scenario_Manager_Ptr = Scenario_Manager::Get_Ptr();

    Assert (Sim_Scheduler_Pt_ == 0, "Krnl_Vis::Initialise - Sim_Scheduler_Pt_ has already been Initialised");
    Sim_Scheduler_Pt_ = Simulation_Scheduler::Get_Ptr();
}
//----------------------------------------------------------------------------------------------------------------
