/****************************************************************************
*
*	  Module Filename:  Realised_Interaction_Type.cpp
*	  Module Name:      Realised_Interaction_Type
*	  Description:      An Interaction Type that is or has been Realised in the Reactor
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include  "globldef.h"
#include  "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//-------------------- Reactor  ----------------------------------------------------------------------------------
#include  "Realised_Interaction_Type.h"
#include  "Realisable_Interaction_Type.h"

//-------------------- Chemistry ---------------------------------------------------------------------------------
#include  "Interaction_Type.h"

//-------------------- Statistics --------------------------------------------------------------------------------
#include  "Simulation_Statistics.h"

//================================================================================================================
       
//----------------------------------------------------------------------------------------------------------------

int   Realised_Interaction_Type::Get_Num_Act_Ints()

//  Return the Number of Actual Interactions for this Realised_Interaction_Type

{   
    return Num_Act_Ints;
}
//----------------------------------------------------------------------------------------------------------------
Indexed_Series*  Realised_Interaction_Type::Get_Series_Ptr()

//  Return a pointer to the Realised_Interaction_Type's Data Series

{
    return Series_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

void  Realised_Interaction_Type::Increment_Num_Act_Ints(int Num_Ints)

//  Increment the number of actual interactions for the Realised Interaction Type by the specified amount

{
    Num_Act_Ints += Num_Ints;
}
//----------------------------------------------------------------------------------------------------------------
Realised_Interaction_Type::Realised_Interaction_Type(Interaction_Type*  Int_Type_Ptr,
                                                     const string& Indexed_Series_Name,
                                                     const string& Index_Name, 
                                                     const string& Value_Name,
                                                     const string& Series_List)
//  Constructor

{

//  Link to the Interaction Type

    Interaction_Type_Ptr = Int_Type_Ptr;
    
//  Create a Data Series for the Realised_Interaction_Type    
    
    string Int_Type_Id = To_String(Interaction_Type_Ptr->Get_Identifier());
    Series_Ptr = Simulation_Statistics::Get_Ptr()->
                    Create_Indexed_Series(Indexed_Series_Name,
                                          Index_Name, 
                                          Value_Name, 
                                          Series_List,
                                          true);
    Num_Act_Ints = 0; 
}
//----------------------------------------------------------------------------------------------------------------

Realised_Interaction_Type::~Realised_Interaction_Type()

{
    Simulation_Statistics::Get_Ptr()->Delete_Series(Series_Ptr);
}
//-----------------------------------------------------------------------------------------------------------------

void   Realised_Interaction_Type::Set_Num_Act_Ints(int Num_Ints)

//  Set the Number of Actual Interactions for the Realised Interaction Type

{
   Num_Act_Ints = Num_Ints;
}
