/****************************************************************************
*
*	  Header Filename:  Realised_Molecule_Type_H
*	  Module Name:      Realised_Molecule_Type
*	  Description:      Molecule_Type that is realised as a Molecule in the Reactor
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Realised_Molecule_Type_H
#define Realised_Molecule_Type_H

class   Realised_Molecule_Type;  // Type specifier used in Molecule_Type
class   Molecule_Type;
class   Molecule;
class   Tracker;

class   Realisable_Interaction_Type;
class   VCSList;
class   Indexed_Series;

class   Realised_Molecule_Type

//  Identifier for a Molecule_Type that is (or has been) realised as a Molecule in the Reactor

{

    friend class Reactor;

    private:
//===== Relationships with other classes ===================================================================

        VCSList*  TList__Of_Molecules_Of_Type_Ptr;    //  List of Molecules in the Reactor of
                                                      //  the realised type

//      ***  APPROACH HERE WORKS, BUT IS NOT CONSISTENT WITH MOLECULE_TYPE CLASS
//      WHICH ALSO HAS A LIST OF INT TYPES WITH THE MOL TYPE AS A PRODUCT.
//      THIS IMPACTS USER CLASSES, WHICH MUST SCAN FULL R INT TYPE LIST
//      FOR REACTOR TO GET A LIST OF R INT TYPES WITH THE MOL TYPE AS PRODUCT.

        DN::List<Realisable_Interaction_Type*>  List__Of_Realisable_Int_Ptrs;
                                                    // List of Realisable Interactions
                                                    // in which the Realised Molecule Type
                                                    // is a potential Catalyst or Reactant

        Molecule_Type*  Molecule_Type__Ptr;   //  Definition of the Molecule_Type
        Indexed_Series*       Series_Ptr;  // Indexed Series used to collect statistics on the Realised_Molecule_Type
//==========================================================================================================        

    public:
//      Public Functions
        ~Realised_Molecule_Type();
        void            Add_Molecule();
        void            Add_Molecule(double  Tracker_Probability);
        void            Add_Molecule(list<Tracker*>* Tracker_List_Ptr);
        void            Add_Molecule(double  Tracker_Probability, list<Tracker*>* Tracker_List_Ptr);

        static  bool    Compare_MolType_Id(const Realised_Molecule_Type* x, const Realised_Molecule_Type* y);
        list<Tracker*>*        Remove_Molecule();
        Molecule_Type*  Get_Molecule_Type_Ptr() const;
        int             Get_Num_Of_Mols_In_Reactor();
        int             Get_Num_Of_R_Int_Types_In_Reactor() const;
        Realisable_Interaction_Type*  Get_Current_R_Int_Type_Ptr();
        DN::List<Realisable_Interaction_Type*>& Get_Int_Type_List_Ref();    
        Indexed_Series* Get_Series_Ptr();   

        Realised_Molecule_Type(Molecule_Type*  MolType_Ptr,
                               const string& Indexed_Series_Name,
                               const string& Index_Name, 
                               const string& Value_Name,
                               const string& Series_List);
    private:
 
    private:
//      Private Functions             


};

//	Declaration of Data Structures constructed by this class only

//      None

//	Defines used with class interface (always prefixed with class name)

//	None


#endif  // Realised_Molecule_Type_H
