/****************************************************************************
*
*     Module Filename:  Scenario_Series.cpp
*	  Module Name:      Scenario Series
*	  Description:      Represents and Manages a series of simulation scenarios
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include "globldef.h"
#include "Library_Headers.h"


#include "globldef.h"
#include "Library_Headers.h"

//--------------  Simulation  Subsystem  ------------------------------------------------------------------------
#include "Scenario_Series.h"
#include "Scenario.h"
#include "ActReq_Requestor.h"
//#include "Action_Request.h"
//---------------------------------------------------------------------------------------------------------------

//--------------  Persistent Data Manager Subsystem  ------------------------------------------------------------
#include "Simulation_Input.h"
#include "Input_Record.h"
//---------------------------------------------------------------------------------------------------------------

Scenario*  Scenario_Series::Get_Curr_Scenario_Ptr()

{
    TRACE("Scenario_Series::Get_Curr_Scenario_Ptr");
    if (Scenario_List.empty()) return 0;
    return (*Current_Scenario_Iter);
}
//---------------------------------------------------------------------------------------------------------------

bool Scenario_Series::Iter_To_Next_Scenario()

//  Move to the next Scenario in the Series

{
    TRACE("Scenario_Series::Iter_To_Next_Scenario");
    if(++Current_Scenario_Iter == Scenario_List.end())
    THEN
        return false;
    ELSE
        return true;
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------

bool  Scenario_Series::Run_Current_Scenario(int Num_Timesteps)

{
    TRACE("Scenario_Series::Run_Current_Scenario");
    bool Reactor_Enabled = (*Current_Scenario_Iter)->Run(Num_Timesteps);
    return Reactor_Enabled;
}
//---------------------------------------------------------------------------------------------------------------

Scenario_Series::Scenario_Series()

{
    TRACE("Scenario_Series::Scenario_Series()");
    CREATE_OBJECT(Scenario, Scenario*  Scen_Ptr = new Scenario);
    Scenario_List.push_back(Scen_Ptr);
    Current_Scenario_Iter = Scenario_List.begin();
}
//----------------------------------------------------------------------------------------------------------------

bool Scenario_Series::Setup_From_Input_File(string Input_File, string&  Error_Text)

//  Setup the Scenario Series from the user input data.
//  1) The input is first read into Input Records and validated for syntax errors
//  2) The Semantics of the Input Records is then validated, and they are used to
//     make and populate Action Request Requestors
//  3) Action Requests are made from the (valid) Requestors

//  In case of any error, return false and pass back Error_Tx

{
    TRACE("Scenario_Series::Setup_From_Input_File");

    list<Input_Record*>  Input_Records;
    list<Input_Record*>* Input_Records_Ptr = &Input_Records;

    list<Action_Request*> ActReqs_To_Schedule;
    list<Action_Request*> ActReqs_In_Scenario;

//  1) Read and validate syntax
    if (not Simulation_Input::Get_Ptr()->Read_Input_Records_And_Validate_Syntax(Input_File, 
                                                                                Input_Records_Ptr, 
                                                                                Error_Text))
    THEN
        return false; 
    ENDIF
    
    int Rcd_Num = 0;
    for(list<Input_Record*>::iterator Rec_Iter = Input_Records_Ptr->begin();
        Rec_Iter not_eq Input_Records_Ptr->end();
        Rec_Iter ++)
    BEGIN
        if (not ((*Rec_Iter)->Empty()))  // Ignore blank records
        THEN   
            ++Rcd_Num;         

//          2) Semantic processing
            ActReq_Requestor*  Rqstr;
            bool  Smntic_Err_Fl = false;

//          Check Record Type
            if (not ActReq_Requestor::Check_RecType_And_Make_Rqstr(*Rec_Iter, 
                                                                   Rqstr, // Will be created
                                                                   Error_Text)) 
                Smntic_Err_Fl = true;
//          Check Record data
            if (not Smntic_Err_Fl )
            THEN
                if (not Rqstr->Proc_Rcd_Semantics(*Rec_Iter, 
                                                Error_Text)) 
                    Smntic_Err_Fl = true;
            ENDIF

            if (Smntic_Err_Fl)
            THEN
                Error_Text = string("Semantic error in record ")
              + To_String(Rcd_Num)
              + string(" - ")
              + (*Rec_Iter)->Get_Record_As_String()
              + string("\n") 
              + Get_ErrLine_Tx()
              + string(" Error is: ")
              + Error_Text
              + string(".");

                Scenario* Scen_Pt = *Current_Scenario_Iter;
                Scen_Pt->Clear();
                return false;            
            ENDIF

//          3) Make two Action Requests and store them for later use. 
            Scenario* Scen_Ptr = *Current_Scenario_Iter;
            Action_Request* Scen_Req_Pt = Rqstr->Make_ActReq();  
            Scen_Ptr->Add_ActReq_To_Scen(Scen_Req_Pt);
            
            Action_Request* Sched_Req_Pt = Rqstr->Make_ActReq();  
            Scen_Ptr->Schedule_ActReq(Sched_Req_Pt);
 
            DELETE_OBJECT(ActReq_Requestor, Rqstr);
        ENDIF
    END
    return true;
}
