/****************************************************************************
*
*	  Module Filename:  Tracker_Manager.cpp
*	  Module Name:      Tracker_Manager
*	  Description:      Manager a Reactor's Trackers
*                      
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*	  Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

//--------------  Tracker  Subsystem  -----------------------------------------------------------------------------

#include  "Tracker_Manager.h"
#include  "Tracker.h"
//----------------------------------------------------------------------------------------------------------------

Tracker_Manager*  Tracker_Manager::Instance_Pt = 0;
//================================================================================================================

int  Tracker_Manager::Number_Of_Trackers;

//---------------------------------------------------------------------------------------------------------------

Tracker* Tracker_Manager::Add_Tracker()

//  Create a Tracker object and add it to the Tracker Manager's list of Trackers

{
    CREATE_OBJECT (Tracker,  Tracker* Tracker_Ptr = new Tracker(this));
    List_Of_Trackers.push_front(Tracker_Ptr);

    Number_Of_Trackers++;
    
//  Get an iterator to the Tracker so that it can be erased quickly later
//  **** RELIES ON ITERATORS TO ITEMS IN A LIST REMAINING OK WHILE SUBSEQUNT OPERATIONS OCCOR  **

    list<Tracker*>::iterator   Iter_To_Inserted_Tracker = List_Of_Trackers.begin();
    Tracker_Ptr->Set_Iter_To_Place_In_List(Iter_To_Inserted_Tracker);

    return Tracker_Ptr;
}
//---------------------------------------------------------------------------------------------------------------

void  Tracker_Manager::Clear()

//  Clear the Tracker Manager

{
    List_Of_Trackers.clear();
    Number_Of_Trackers = 0;
}
//----------------------------------------------------------------------------------------------------------------

int    Tracker_Manager::Get_Num_Trackers()

//  Return the number of Trackers in the Reactor

{
    return Number_Of_Trackers;
}
//---------------------------------------------------------------------------------------------------------------

Tracker_Manager*  Tracker_Manager::Get_Pt()

{
    TRACE("Tracker_Manager::Get_Pt");

    Assert (Instance_Pt != 0, "Tracker_Manager::Get_Pt - Instance Pointer not initialised");
    return Instance_Pt;
}
//---------------------------------------------------------------------------------------------------------------
 
void   Tracker_Manager::List_Trackers()

//  TESTING ONLY

{
    cout << "Trackers:  " << endl;
    list<Tracker*>::iterator  Iter;
    for(Iter = List_Of_Trackers.begin();
        Iter not_eq List_Of_Trackers.end();
        Iter++)
    BEGIN
        string Path;
        (*Iter)->Get_Path_As_String(Path);
        cout <<  Path << endl;
    END
}

//---------------------------------------------------------------------------------------------------------------
void   Tracker_Manager::Remove_Tracker(Tracker*  Tracker_Ptr)

//  Unlink the specified Tracker from the List of Trackers and then delete the Tracker
//  Check that the Trackers Iterator to its place in the List is OK (ie iterator is still valid).

{
    list<Tracker*>::iterator Iter_To_List_Position = Tracker_Ptr->Get_Iter_To_Place_In_List();
    Assert(*Iter_To_List_Position == (Tracker_Ptr), "Tracker_Manager::Remove_Tracker - Tracker's Iterator to its place in the Reactor's List of Trackers does not refer to the same object as the list item itself");

    List_Of_Trackers.erase(Iter_To_List_Position);
    Tracker::Destroy(Tracker_Ptr);

    Number_Of_Trackers--;

    Assert (Number_Of_Trackers >= 0, "Tracker_Manager::Remove_Tracker - Negative Number of Trackers!");
}
//---------------------------------------------------------------------------------------------------------------

Tracker_Manager::Tracker_Manager(Realised_Cycle_Type_Manager*  RCT_Mgr_Ptr)

//  Constructor

{
    RCT_Manager_Ptr = RCT_Mgr_Ptr;
    Number_Of_Trackers = 0;
    Instance_Pt = this;
}


