/****************************************************************************
*
*     Module Filename:  Input_Record.h
*     Module Name:      Input_Record
*     Description:      Input Record handler: Holds data from a record of input
*                       read from a file, and makes it available, with 
*                       with validations and conversions, throug an API
*
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2008, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*      Amendment History:- 
*
*              Version	  Date                        Update Description
*              =======    ===========                 ==================
*
*
*
*
****************************************************************************/

#ifndef Input_Record_H
#define Input_Record_H

class   Input_Record

{
//===== Relationships with other classes ===================================================================
  
//==========================================================================================================

    public:

        bool Check_All_Attributes_Read(string&  Error_Text);
        bool Check_No_Attrib_Strings_Left(string Attrib_Name,  
                                          list<string>& Attrib_Strings,   
                                          string& Error_Text);
        bool Empty () {return Record_Empty;};

//      Attribute Access. Pass Attribute Name as string rather than as code.
//      This avoids making Input_Record (which is responsible for syntax) 
//      de[endent on knowledge of attribute codes (which are part of semanric analysis) 
        bool Get_Attribute_Strings(string Attrib_Name,  
                                   list<string>& Attrib_Strings,     
                                   string& Error_Text);
        bool Get_Attr_As_PosInt(string  Attrib_Name, 
                                int& Value, 
                                string&  Error_Text);
        bool Get_Attr_As_NonNegInt(string  Attrib_Name, 
                                   int& Value, 
                                   string&  Error_Text);
        bool Get_Attr_As_Rand_Or_Float(string  Attrib_Name,
                                       bool& Rand_Flag,
                                       double&  Value,
                                       string&  Error_Text);
        bool Get_Attr_As_Rand_Or_NonNegInt(string  Attrib_Name,
                                           bool& Rand_Flag,
                                           int& Value, 
                                           string&  Error_Text);
        bool Get_Attr_As_Rand_Or_PosInt(string  Attrib_Name,
                                        bool& Rand_Flag,
                                        int& Value, 
                                        string&  Error_Text);
        bool Get_Attr_As_String(string  Attrib_Name, 
                                string& Value, 
                                string&  Error_Text);

        string Get_Record_Type_Text();
        string Get_Record_As_String();
        void Print(int Record_Num);
        bool Read_Record_And_Validate_Syntax(istream& Input_Stream, 
                                             int  Record_Num,
                                             bool& EOF_Flag, 
                                             string& Error_Text);      
        
    private:
        bool    Get_Single_PosInt_Value(int& Value, string&  Error_Text);
        bool    Read_Record(istream& Input_Stream, bool& EOF_Flag, string&  Error_Text);
        list <string>  Input_Record;  // Working data
        list <string>  Input_Record_For_Print;
        string  Record_Type_Text;
        map <string, list<string> > Attribs_And_Data;   // Keys are attribute names
                                                        // Values are the input data.
                                                        // Data for each attribule can consist of several strings
        list <string> Attribs_Read;
        bool Record_Empty;   // Record read from file was empty (all white space)
};
#endif  //Input_Record_H
