/****************************************************************************
*
*	  Header Filename:  TForm_View_Network.h
*	  Module Name:      TForm_View_Network
*	  Description:      View Network
*	  Author:   		Chris Gordon-Smith
*
*     Copyright (C) 2003, 2004, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*     Version	   Date	   			Update Description
*     =======    ===========		==================
*
*
*
*
****************************************************************************/

#ifndef Form_View_Network_H
#define Form_View_Network_H

//--------------  GUI Presentation Subsystem API ----------------------------------------------------------------
#include  "GUI_Presentation_API.h"
//---------------------------------------------------------------------------------------------------------------

#include  "Network_Painter_Prot.h"  //  Aggregation class

class TForm__View_Network //: public TForm
{

//===== Relationships with other classes =====================================================================================

//    See below for actual code.  IDE gets confused if the real declarations are located here.  
protected:
    Network_Painter_Prot*        Network_Painter_Prot__Ptr;
//  Selected_Node_Table    Selected_Nodes;    //  See below for declaration
//==========================================================================================================================

    void FormClose();
    void TListView__Connections_KeyUp();
    void Menu_R_MolTyp_List_Add_Mol();
    void Menu_R_MolTyp_List_Rem_Mol();
    void Menu_MolTyp_List_Add_Mol();
    void Menu_MolTyp_List_Refresh();
    void Menu_R_MolTyp_List_Refresh();
    void List_Interactions_Click();

public:

//  Public Functions
//        void Clear();
        void Set_Number_Of_Nodes(int Num_Nodes);

        TForm__View_Network();
        virtual  ~TForm__View_Network(){};  

        void   Display_All();
        void   Display_Network();
        void   Clear_Network_Bitmap();

    private:

protected:
//      Class Data
        

    class Selected_Node_Table
    {

//   Class to manage Selected Nodes

        private:
            map<int,Enum_Node_Selection_State>  Table_Of_Selected_Nodes;
            map<int,Enum_Node_Selection_State>::iterator  Iter__Table_Of_Selected_Nodes;

//          Second Iterator.  
            map<int,Enum_Node_Selection_State>::iterator  Iter2__Table_Of_Selected_Nodes;

            int  Node_Index_At_Mouse_Down;

        public:
            void  Clear();
            void  Select_Node(int Node_Index, Enum_Node_Selection_State  Selection_State);
            void  Deselect_Node(int  Node_Index);
            void  Deselect_All_Nodes();                 
            Enum_Node_Selection_State   Get_Node_Selection_State(int Node_Index);
            int   Get_Node_Index_At_Mouse_Down(void) const;
            void  Set_Node_Index_At_Mouse_Down(int  Node_Index);
            bool  Iter_To_First();
            bool  Iter_To_Next();
            int   Get_Current_Node() const;
    };

    Selected_Node_Table    Selected_Nodes;

    bool    Last_MouseDown_Was_Left_Mouse;

protected:
//    Functions

    void    Initialise_Form(int Number_Of_Nodes, 
                            bool Display_All_Connections_Ind,  
                            Widget_Ids::Drawing_Area_Id  Draw_Area_Id);

    virtual void    Close_View() = 0;

    virtual void    Add_Node() const {};
    virtual void    Add_Item_Of_Node_Type() const {};

    virtual void    Remove_Node() const {};
    virtual void    Remove_Item_Of_Node_Type() const {};


//    virtual void    List_Connections_For_Node(void* Node_Ptr) const = 0;
    virtual void    List_Nodes() const = 0;

    virtual void    Draw_Connection(void* Void__Connection_Ptr, int Conn_Selection_State) const =0;
    virtual void    Draw_Node(void* Void__Node_Ptr) const = 0;

    void            Draw_Selected_Connections();
    virtual void    Draw_All_Connections() const = 0;
    virtual void    Draw_All_Nodes() const = 0;
    virtual void    Draw_Text_Output() const = 0;

    virtual int     Compare_Nodes(void*  Node1_Ptr, void* Node2_Ptr) const = 0;
    virtual int     Compare_Connections(void*  Connect1_Ptr, void* Connect2_Ptr) const = 0;

    virtual void    Get_Conns_Matching_Node_And_Type
                                   (list<void*>& List_Of_Connections,
                                    int  Node_Index,
                                    Enum_Node_Selection_State  Selection_State) = 0;

    virtual int     Get_Number_Of_Possible_Nodes() = 0;

    void            Get_Conns_Matching_All_Sel_Nodes(list<void*>& List__Of_Connections);

    virtual void    Draw_Selected_Nodes() = 0;
    virtual void    Draw_Connections_For_Selected_Nodes();
};


#endif  //Form_View_Network_H

