/****************************************************************************
*
*     Module Filename:  Drawing_Area.cpp
*     Module Name:      Drawing_Area
*     Description:      Drawing Area for SimSoup Display
*                      
*     Author:           Chris Gordon-Smith
*
*     Copyright (C) 2007 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License as published by the
*     Free Software Foundation; either version 2 of the License, or (at your
*     option) any later version.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*     Amendment History:-
*
*         Version    Date                 Update Description
*         =======    ===========          ==================
*
*
*
*
****************************************************************************/

#include    "globldef.h"
#include    "Library_Headers.h"

#include    <gtkmm.h>

//-------------- User Interface Presentation Subsystem  ----------------------------------------------------------
#include  "Drawing_Area.h"
#include  "Colour.h"
//----------------------------------------------------------------------------------------------------------------

void  Drawing_Area::Clear()

//  Clear the Drawing Area

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        GDK_Context->set_foreground(Colour::White());
        GDK_Pixmap->draw_rectangle(GDK_Context, true, 0,0, 1400,1200);  
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

Drawing_Area::Drawing_Area()

//  Constructor

{
    Realised = false;
}  
//----------------------------------------------------------------------------------------------------------------

bool Drawing_Area::on_expose_event(GdkEventExpose* event)

//  Handler for On Expose Event

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
//  Draw / Redraw the Pixmap

        GDK_Window->draw_drawable (GDK_Context, GDK_Pixmap, 0,0, 0, 0);
    ENDIF
    return true;
}  
//----------------------------------------------------------------------------------------------------------------

void  Drawing_Area::Draw_Line(int X1, int Y1, int X2, int Y2)

//  Draw the specified line on the Pixmap using the current GDK Context

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        GDK_Context->set_line_attributes(2,                 //line_width
                                        Gdk::LINE_SOLID,   //LineStyle
                                        Gdk::CAP_NOT_LAST, //CapStyle
                                        Gdk::JOIN_MITER);  //JoinStyle

        GDK_Pixmap->draw_line(GDK_Context,X1,Y1, X2, Y2);  
    ENDIF
}
//-----------------------------------------------------------------------------------------------------------------

void Drawing_Area::Draw_Pixmap()

//  Draw the Pixmap on the GDK Window

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        GDK_Window->draw_drawable (GDK_Context, GDK_Pixmap, 0,0, 0, 0);
    ENDIF
}
//---------------------------------------------------------------------------------------------------------------
   
void Drawing_Area::Draw_Rectangle(bool Filled, int X, int  Y, int Width, int Height)

//  Draw a rectangle on the Pixmap using the current GDK Context

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        GDK_Pixmap->draw_rectangle (GDK_Context, Filled, X, Y, Width, Height);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void Drawing_Area::Get_Size (int& Width, int&  Height)

//  Get the size of the GDK Window

{
    Assert(Realised not_eq 0,"Drawing_Area::Get_Size - Drawing Area is not Realised");
    GDK_Window->get_size(Width, Height);
}
//----------------------------------------------------------------------------------------------------------------

void  Drawing_Area::on_realize()

//  Initialisation logic that can only be run once the Drawing Area has been realised

{
//  We need to call the base on_realize()
    Gtk::DrawingArea::on_realize();

//  Setup the GDK Window, the GDK Pixmap and the GDK Context 

    GDK_Window = get_window();

    GDK_Pixmap = Gdk::Pixmap::create(GDK_Window, 1400,1200,-1);
    GDK_Context = Gdk::GC::create(GDK_Pixmap);

    Realised = true;
    Clear();
}  
//----------------------------------------------------------------------------------------------------------------

void Drawing_Area::Set_Foreground_Colour(Gdk::Color  GDK_Colour)

//  Set the Foreground Colour for drawing to the specified Gdk::Color

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        if (GDK_Context)
        THEN
            GDK_Context->set_foreground(GDK_Colour);
        ENDIF
    ENDIF
}
//----------------------------------------------------------------------------------------------------------------

void  Drawing_Area::Set_Foreground_Colour(int Red, int Green, int Blue)

//  Set the Foreground Colour for drawing to the specified RGB combination

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        GDK_Context->set_foreground(Colour::Get_GDK_Colour_RGB(Red, Green, Blue));
    ENDIF
}
//-----------------------------------------------------------------------------------------------------------------

void Drawing_Area::Text_Out(int X, int Y, string  Text)

//  Write the specified text to the specified location on the Pixmap

{
    if (Realised not_eq 0)  // Check that Drawing Area has been Realised
    THEN
        Glib::RefPtr<Pango::Layout>  Layout = create_pango_layout(Text);
        GDK_Pixmap->draw_layout(GDK_Context,X,Y, Layout);  
    ENDIF
}


